/************************************************************************************
 *  File:     Flash.ld
 *  Purpose:  Linker script to configure memory regions for RX111
 *            and place sections and symbol values.
 *  Date:     26 September 2013
 *  Info:     Reset_Handler must be defined in code as entry point of reset handler.
 *
 *            By default the script is configured for the first code flash and RAM bank
 *            of the device.
 *            If additional banks of the device shall be used,
 *            they can be configured manually.
 *
 *            Following symbols are defined in the script
 *            and can be used without definition in code:
 *
 ************************************************************************************/

OUTPUT_ARCH(rx)

/************************************************************************************
 *
 *  Memory regions and required amount of heap and stack
 ************************************************************************************/
MEMORY
{
  FLASH1 (rx)      : ORIGIN = 0xFFE00000, LENGTH = 0x00200000
  RAM (xrw)        : ORIGIN = 0x00000000, LENGTH = 0x00020000
}
_Min_Heap_Size  = 0x800;   /* required amount of heap  */
_Min_Stack_Size = 0x400;  /* required amount of stack */

/************************************************************************************
 *
 *  Code and data sections
 ************************************************************************************/
ENTRY(_Reset_Handler)

SECTIONS
{
/************************************************************************************
 *
 *  vector section (code)
 *  Section for interrupt vectors
 *  Goes into FLASH
 ************************************************************************************/
  .fvectors 0xFFFFFF80 :
  {
    KEEP(*(.fvectors))
  }
/************************************************************************************
 *
 *  text section (code)
 *  Starts with startup code
 *  Goes into FLASH
 ************************************************************************************/
  .text :
  {
    /* Code section */
    *(.text)
    *(.text.*)
    *(P)

    /* Global symbol at the end of code (text section) */
    etext = .;
  } > FLASH1

/************************************************************************************
 *
 *  Relocatable vector section
 *  Goes into FLASH
 ************************************************************************************/
  .rvectors :
  {
    _rvectors_start = .;
    KEEP(*(.rvectors))
    _rvectors_end = .;
  } > FLASH1

/************************************************************************************
 *
 *  Initialized data
 *  Goes into FLASH
 ************************************************************************************/
  .init :
  {
    *(.init)
    __preinit_array_start = .;
    *(.preinit_array)
    __preinit_array_end = .;

    __init_array_start = (. + 3) & ~ 3;
    *(.init_array)
    *(SORT(.init_array.*))
    __init_array_end = .;

    __fini_array_start = .;
    *(.fini_array)
    *(SORT(.fini_array.*))
    __fini_array_end = .;
  } > FLASH1

  .fini :
  {
    *(.fini)
  } > FLASH1

  .got :
  {
    *(.got)
    *(.got.plt)
  } > FLASH1

/************************************************************************************
 *
 *  Read-only data section
 *  (constants, strings, ...)
 *  Goes into FLASH
 ************************************************************************************/
  .rodata :
  {
    *(.rodata)
    *(.rodata.*)
    *(C_1)
    *(C_2)
    *(C)
    _erodata = .;
  } > FLASH1

  .eh_frame_hdr :
  {
    *(.eh_frame_hdr)
  } > FLASH1

  .eh_frame :
  {
    *(.eh_frame)
  } > FLASH1

  .jcr :
  {
    *(.jcr)
  } > FLASH1

  .tors :
  {
    __CTOR_LIST__ = .;
    __ctors = .;
    *(.ctors);
    __ctors_end = .;
    __CTOR_END__ = .;

    __CTOR_LIST__ = .;
    __dtors = .;
    *(.dtors);
    __dtors_end = .;
    __DTOR_END__ = .;
  } > FLASH1

  /* Global symbol at the end of code (text section) */
  _mdata = .;

 /************************************************************************************
 *
 *  data section
 *  Goes into RAM
 ************************************************************************************/
  .data : AT (_mdata)
  {
    _data = .;
    *(.data)
    *(.data.*)
    *(D)
    *(D_1)
    *(D_2)
    _edata = .;
  } > RAM

  .gcc_exc :
  {
    *(.gcc_exc)
  } > RAM

/************************************************************************************
 *
 *  bss section (Uninitialized data)
 *  Goes into RAM
 ************************************************************************************/
  .bss :
  {
    _bss = .;
    *(.bss)
    *(.bss.*)
    *(COMMON)
    *(B)
    *(B_1)
    *(B_2)
    _ebss = .;
    _end = .;
  } > RAM

/************************************************************************************
 *
 *  heap_stack section
 *  Check that there is enough RAM left for stack and heap
 ************************************************************************************/
  .heap_stack :
  {
    . = ALIGN(8);
    __heap_start = .;
    . = . + _Min_Heap_Size;
    __heap_end = .;
    . = ALIGN(8);
    __ustack_start = .;
    . = . + _Min_Stack_Size;
    _ustack = .;
    . = ALIGN(8);
    __istack_start = .;
    . = . + 0x200;
    _istack = .;
  } > RAM

}
